from django.core.management.base import BaseCommand
from bulbs.sections.models import Section


class Command(BaseCommand):
    help = 'Indexes all section objects in the .percolator document'

    def handle(self, *args, **options):
        for section in Section.objects.all():
            if section.query is not None or section.query != {}:
                section._save_percolator()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0005_content_instant_article'),
    ]

    operations = [
        migrations.AddField(
            model_name='content',
            name='tunic_campaign_id',
            field=models.IntegerField(default=None, blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import json_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('content', '0001_initial'),
        ('promotion', '0001_initial'),
    ]

    operations = [

        # cleanup old models
        migrations.DeleteModel(
            name='LockOperation',
        ),
        migrations.DeleteModel(
            name='UnlockOperation',
        ),
        migrations.DeleteModel(
            name='InsertOperation'
        ),
        migrations.DeleteModel(
            name='ReplaceOperation'
        ),
        migrations.DeleteModel(
            name='ContentListOperation'
        ),
        migrations.DeleteModel(
            name='ContentListHistory',
        ),

        # fix up content list which is now pzone
        migrations.RenameModel(
            old_name='ContentList',
            new_name='PZone',
        ),
        migrations.RenameField(
            model_name='pzone',
            old_name='length',
            new_name='zone_length',
        ),

        # pzone operation modifications
        migrations.CreateModel(
            name='PZoneOperation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('when', models.DateTimeField()),
                ('applied', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ['-when', 'id'],
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='pzoneoperation',
            name='content',
            field=models.ForeignKey(related_name='+', to='content.Content'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='pzoneoperation',
            name='polymorphic_ctype',
            field=models.ForeignKey(related_name='polymorphic_promotion.pzoneoperation_set', editable=False, to='contenttypes.ContentType', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='pzoneoperation',
            name='pzone',
            field=models.ForeignKey(related_name='operations', to='promotion.PZone'),
            preserve_default=True,
        ),

        # delete operation modifications
        migrations.CreateModel(
            name='DeleteOperation',
            fields=[
                ('pzoneoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.PZoneOperation')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.pzoneoperation',),
        ),

        # insert operation modifications
        migrations.CreateModel(
            name='InsertOperation',
            fields=[
                ('pzoneoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.PZoneOperation')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.pzoneoperation',),
        ),
        migrations.AddField(
            model_name='insertoperation',
            name='index',
            field=models.IntegerField(default=0),
            preserve_default=True,
        ),

        # replace operation modifications
        migrations.CreateModel(
            name='ReplaceOperation',
            fields=[
                ('pzoneoperation_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='promotion.PZoneOperation')),
            ],
            options={
                'abstract': False,
            },
            bases=('promotion.pzoneoperation',),
        ),
        migrations.AddField(
            model_name='replaceoperation',
            name='index',
            field=models.IntegerField(default=0),
            preserve_default=True,
        ),

        # pzone history modifications
        migrations.CreateModel(
            name='PZoneHistory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('data', json_field.fields.JSONField(default=[], help_text='Enter a valid JSON object')),
                ('date', models.DateTimeField(auto_now_add=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='pzonehistory',
            name='pzone',
            field=models.ForeignKey(related_name='history', to='promotion.PZone'),
            preserve_default=True,
        ),
    ]



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.utils.fields import RichTextField


class BaseEntrySerializer(serializers.Serializer):

    def __init__(self, *args, **kwargs):
        self.child_label = kwargs.pop("child_label", None)
        super(BaseEntrySerializer, self).__init__(*args, **kwargs)


class CopySerializer(serializers.Serializer):
    copy = RichTextField(required=True, field_size="long")


class EntrySerializer(BaseEntrySerializer, CopySerializer):
    title = RichTextField(required=False, field_size="short")
    image = ImageFieldSerializer(required=False, default=None, allow_null=True)



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.content.models import FeatureType
from bulbs.content.serializers import FeatureTypeField


class RecircContentSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    thumbnail = ImageFieldSerializer(allow_null=True, read_only=True)
    slug = serializers.CharField()
    title = serializers.CharField()
    feature_type = FeatureTypeField(queryset=FeatureType.objects.all())



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from .models import Notification


class NotificationSerializer(serializers.ModelSerializer):
    """Basic model serializer for notifications.Notification."""

    image = ImageFieldSerializer(required=False, allow_null=True)

    class Meta:
        model = Notification



from importlib import import_module

from django.conf import settings


def get_superfeature_model():
    module = getattr(settings, "BULBS_SUPERFEATURE_MODEL_MODULE", None)
    model = getattr(settings, "BULBS_SUPERFEATURE_MODEL", None)
    if model is None or module is None:
        from bulbs.super_features.models import BaseSuperFeature
        return BaseSuperFeature
    else:
        # NOTE: import serializer module & then evaluate based on serializer name
        mod = import_module(module)
        return getattr(mod, model)


def get_superfeature_serializer():
    module = getattr(settings, "BULBS_SUPERFEATURE_SERIALIZER_MODULE", None)
    serializer = getattr(settings, "BULBS_SUPERFEATURE_SERIALIZER", None)
    if serializer is None or module is None:
        from bulbs.super_features.serializers import BaseSuperFeatureSerializer
        return BaseSuperFeatureSerializer
    else:
        # NOTE: import serializer module & then evaluate based on serializer name
        mod = import_module(module)
        return getattr(mod, serializer)


def get_superfeature_partial_serializer():
    module = getattr(settings, "BULBS_SUPERFEATURE_SERIALIZER_MODULE", None)
    serializer = getattr(settings, "BULBS_SUPERFEATURE_PARTIAL_SERIALIZER", None)
    if serializer is None or module is None:
        from bulbs.super_features.serializers import BaseSuperFeaturePartialSerializer
        return BaseSuperFeaturePartialSerializer
    else:
        # NOTE: import serializer module & then evaluate based on serializer name
        mod = import_module(module)
        return getattr(mod, serializer)


def get_superfeature_metadata():
    module = getattr(settings, "BULBS_SUPERFEATURE_METADATA_MODULE", None)
    metadata = getattr(settings, "BULBS_SUPERFEATURE_METADATA", None)
    if metadata is None or module is None:
        from bulbs.super_features.metadata import BaseSuperFeatureMetadata
        return BaseSuperFeatureMetadata
    else:
        # NOTE: import metadata module & then evaluate based on metadata name
        mod = import_module(module)
        return getattr(mod, metadata)


def get_superfeature_choices():
    from bulbs.super_features.models import BASE_CHOICES

    configured_superfeatures = getattr(settings, "BULBS_CUSTOM_SUPERFEATURE_CHOICES", ())
    for sf in configured_superfeatures:
        if type(sf[0]) is not str or type(sf[1]) is not str:
            raise ValueError(
                "Super Feature choices must be a string. {0} is not valid".format(sf)
            )
    return BASE_CHOICES + configured_superfeatures



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import djbetty.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='VideohubVideo',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('title', models.CharField(max_length=512)),
                ('description', models.TextField(blank=True, default='')),
                ('keywords', models.TextField(blank=True, default='')),
                ('image', djbetty.fields.ImageField(blank=True, null=True, alt_field='_image_alt', default=None, caption_field='_image_caption')),
                ('_image_alt', models.CharField(blank=True, null=True, max_length=255, editable=False)),
                ('_image_caption', models.CharField(blank=True, null=True, max_length=255, editable=False)),
                ('channel_id', models.IntegerField(blank=True, null=True, default=None)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0012_auto_20160615_1605'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseInfographic',
            fields=[
                ('content_ptr', models.OneToOneField(primary_key=True, to='content.Content', parent_link=True, auto_created=True, serialize=False)),
                ('infographic_type', models.IntegerField(default=0)),
                ('data', jsonfield.fields.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
            bases=('content.content',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('campaigns', '0005_campaign_tunic_campaign_id'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='campaignpixel',
            name='campaign',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import djbetty.fields


class Migration(migrations.Migration):

    dependencies = [
        ('special_coverage', '0009_auto_20160208_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='specialcoverage',
            name='image',
            field=djbetty.fields.ImageField(caption_field='_image_caption', default=None, null=True, alt_field='_image_alt', blank=True),
        ),
    ]



from django import template

register = template.Library()


@register.inclusion_tag('videos/video-recirc.html', takes_context=True)
def video_recirc_widget(context, source, count):
    context["source"] = source
    context["count"] = count
    return context



from django.contrib.auth import get_user_model

from elasticsearch_dsl import field


class ContributorField(field.Object):

    def __init__(self, *args, **kwargs):
        super(ContributorField, self).__init__(*args, **kwargs)
        self.properties['username'] = field.String(index='not_analyzed')
        self.properties['is_freelance'] = field.Boolean()

    def to_es(self, obj):
        data = {
            'id': obj.id,
            'username': obj.username,
            'first_name': obj.first_name,
            'last_name': obj.last_name
        }
        profile = getattr(obj, 'freelanceprofile', None)
        if profile:
            data['is_freelance'] = profile.is_freelance
        return data

    def to_python(self, data):
        User = get_user_model()
        user = User.objects.filter(id=data['id'])
        if user.exists():
            return user.first()


class ContentContributionsField(field.Nested):
    """
    This needs a better long-term solution, but alas, time is cruel.
    """

    def __init__(self, *args, **kwargs):
        super(ContentContributionsField, self).__init__(*args, **kwargs)
        self.properties['contributor'] = ContributorField()

    def to_es(self, obj):
        data = {}
        return data

    def to_dict(self, **kwargs):
        return {}



from rest_framework import serializers


class GlanceContentSerializer(serializers.Serializer):

    type = serializers.IntegerField(required=True, allow_null=False)
    id = serializers.CharField()
    title = serializers.CharField()
    link = serializers.URLField()
    modified = serializers.DateTimeField()
    published = serializers.DateTimeField()
    slug = serializers.CharField()
    images = serializers.DictField(
        child=serializers.URLField()
    )
    authors = serializers.ListField(
        child=serializers.CharField()
    )
    tags = serializers.DictField(
        child=serializers.ListField(
            child=serializers.CharField()))

    def to_representation(self, obj):
        return {
            'type': 'post',
            'id': obj.id,
            'title': obj.title,
            'modified': obj.last_modified.isoformat(),
            'published': obj.published.isoformat(),
            'slug': obj.slug,
            'images': {
                'post-16-9-thumbnail': obj.thumbnail.get_crop_url(ratio='16x9'),
            },
            'link': self.context['request'].build_absolute_uri(obj.get_absolute_url()),
            # mparent(2016-05-04) TODO: Optional author support
            'authors': ["America's Finest News Source"],
            'tags': {
                'section': [tag.name for tag in obj.ordered_tags()],
            },
        }



from django.apps import apps
from django.conf import settings
from django.core.exceptions import FieldError
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.decorators.cache import cache_control


@cache_control(max_age=3600)
def video_redirect(request, videohub_ref_id):
    video_model_name = getattr(settings, "VIDEO_MODEL", "")
    video_model = apps.get_model(video_model_name)
    try:
        video = get_object_or_404(video_model, videohub_ref__id=int(videohub_ref_id))
    except FieldError:
        raise Http404("No video found with the provided videohub_ref id.")
    return HttpResponseRedirect(video.get_absolute_url())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0007_auto_20160122_1459'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lineitem',
            name='contributor',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='line_items'),
        ),
    ]
